use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007002
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
ObjectID
PCRE
VERSIONING
bson
codec
dbrefs
ixhash
numification
oid
sharding
tock
David
Golden
david
Stefan
minimalist
and
MongoDB
Inc
Eric
Daniels
eric
Pat
Gunn
pgunn
Yury
Zavarin
yury
Oleg
Kostyuk
cub
lib
BSON
Binary
Bool
Bytes
Code
DBRef
Decimal128
Doc
Double
Int32
Int64
MaxKey
MinKey
OID
ObjectId
PP
Raw
Regex
String
Time
Timestamp
Types
